@@ -168,7 +168,7 @@ static void init(void)
 	mpg123_cfg.proxy_user = NULL;
 	mpg123_cfg.proxy_pass = NULL;
 	mpg123_cfg.cast_title_streaming = FALSE;
-	mpg123_cfg.use_udp_channel = TRUE;
+	mpg123_cfg.use_udp_channel = FALSE;
 	mpg123_cfg.title_override = FALSE;
 	mpg123_cfg.disable_id3v2 = FALSE;
 	mpg123_cfg.detect_by = DETECT_EXTENSION;
