@@ -41,13 +41,14 @@
 static int read_all(int fd, void *buf, size_t count)
 {
 	size_t left = count;
+	int r;
 	do {
-		int r = read(fd, buf, left);
+		r = read(fd, buf, left);
 		if (r < 0)
 			return -1;
 		left -= r;
 		buf = (char *)buf + r;
-	} while (left > 0);
+	} while (left > 0 && r > 0);
 	return count;
 }
 
@@ -275,7 +276,7 @@ gint xmms_connect_to_session(gint session)
 		stored_uid = getuid();
 		euid = geteuid();
 		setuid(euid);
-		sprintf(saddr.sun_path, "%s/xmms_%s.%d", g_get_tmp_dir(), g_get_user_name(), session);
+		g_snprintf(saddr.sun_path, 108, "%s/xmms_%s.%d", g_get_tmp_dir(), g_get_user_name(), session);
 		setreuid(stored_uid, euid);
 		if (connect(fd, (struct sockaddr *) &saddr, sizeof (saddr)) != -1)
 			return fd;
