@@ -25,7 +25,6 @@
  * to something more sophisticated
  */
 
-extern gint bitrate, frequency, numchannels;
 extern HSlider *mainwin_balance;
 extern TButton *mainwin_repeat, *mainwin_shuffle;
 
@@ -139,7 +138,7 @@ void start_ctrlsocket(void)
 	pthread_mutex_unlock(&start_mutex);
 }
 
-static int write_all(int fd, const void *buf, size_t count)
+int write_all(int fd, const void *buf, size_t count)
 {
 	size_t left = count;
 	/* FIXME: This can take forever */
@@ -158,10 +157,12 @@ static int write_all(int fd, const void *buf, size_t count)
 	return count;
 }
 
-static void ctrl_write_packet(gint fd, gpointer data, gint length)
+void ctrl_write_packet(gint fd, gpointer data, gint length)
 {
 	ServerPktHeader pkthdr;
 
+	/* Not strictly necessary, but zeroes padding */
+	memset(&pkthdr, 0, sizeof (pkthdr));
 	pkthdr.version = XMMS_PROTOCOL_VERSION;
 	pkthdr.data_length = length;
 	if (write_all(fd, &pkthdr, sizeof (ServerPktHeader)) < 0)
@@ -336,12 +337,16 @@ void *ctrlsocket_func(void *arg)
 				ctrl_ack_packet(pkt);
 				break;
 			case CMD_GET_INFO:
-				info[0] = bitrate;
-				info[1] = frequency;
-				info[2] = numchannels;
+			{
+				int rate, freq, chn;
+				mainwin_get_song_info(&rate, &freq, &chn);
+				info[0] = rate;
+				info[1] = freq;
+				info[2] = chn;
 				ctrl_write_packet(pkt->fd, info, 3 * sizeof (gint));
 				ctrl_ack_packet(pkt);
 				break;
+			}
 			case CMD_GET_EQ_DATA:
 			case CMD_SET_EQ_DATA:
 				/* obsolete */
