@@ -1,7 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
- *  Copyright (C) 1998-2002  Peter Alm, Mikael Alm, Olle Hallnas,
+ *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2002  Haavard Kvaalen
+ *  Copyright (C) 1999-2004  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -58,13 +58,13 @@ WinLayer;
 
 static void (*set_always_func)(GtkWidget *, gboolean);
 static void (*set_sticky_func)(GtkWidget *, gboolean);
-static void (*set_skip_winlist_func)(GtkWidget *);
+static void (*set_skip_taskbar_func)(GtkWidget *);
 static void (*move_resize_func)(GtkWidget *, int, int, gboolean);
 
 void hint_set_skip_winlist(GtkWidget *window)
 {
-	if (set_skip_winlist_func)
-		set_skip_winlist_func(window);
+	if (set_skip_taskbar_func)
+		set_skip_taskbar_func(window);
 }
 
 void hint_set_always(gboolean always)
@@ -94,7 +94,7 @@ void hint_set_sticky(gboolean sticky)
 
 gboolean hint_move_resize_available(void)
 {
-	return !!move_resize_func;
+	return 0;
 }
 
 void hint_move_resize(GtkWidget *window, int x, int y, gboolean move)
@@ -102,12 +102,12 @@ void hint_move_resize(GtkWidget *window, int x, int y, gboolean move)
 	move_resize_func(window, x, y, move);
 }
 
-static void gnome_wm_set_skip_winlist(GtkWidget *widget)
+static void gnome_wm_set_skip_taskbar(GtkWidget *widget)
 {
 	long data[1];
 	Atom xa_win_hints = gdk_atom_intern("_WIN_HINTS", FALSE);
 
-	data[0] = WIN_HINTS_SKIP_WINLIST | WIN_HINTS_SKIP_TASKBAR;
+	data[0] = WIN_HINTS_SKIP_TASKBAR;
 	XChangeProperty(GDK_DISPLAY(), GDK_WINDOW_XWINDOW(widget->window),
 			xa_win_hints, XA_CARDINAL, 32,
 			PropModeReplace, (unsigned char *) data, 1);
@@ -320,8 +320,6 @@ static void net_wm_set_desktop(GtkWidget * window, gboolean all)
 		   SubstructureNotifyMask, &xev);
 }
 
-
-
 static void net_wm_set_window_always(GtkWidget * window, gboolean always)
 {
 	net_wm_set_property(window, "_NET_WM_STATE_STAYS_ON_TOP", always);
@@ -332,6 +330,11 @@ static void net_wm_set_window_above(GtkWidget * window, gboolean always)
 	net_wm_set_property(window, "_NET_WM_STATE_ABOVE", always);
 }
 
+static void net_wm_set_skip_taskbar(GtkWidget * window)
+{
+	net_wm_set_property(window, "_NET_WM_STATE_SKIP_TASKBAR", TRUE);
+}
+
 static void net_wm_move_resize(GtkWidget *window, int x, int y, gboolean move)
 {
 	XEvent xev;
@@ -410,6 +413,9 @@ static void net_wm_check_features(void)
 		if (!set_sticky_func &&
 		    find_atom(atoms, n_atoms, "_NET_WM_DESKTOP"))
 			set_sticky_func = net_wm_set_desktop;
+		if (!set_skip_taskbar_func &&
+		    find_atom(atoms, n_atoms, "_NET_WM_STATE_SKIP_TASKBAR"))
+			set_skip_taskbar_func = net_wm_set_skip_taskbar;
 	}
 
 	if (find_atom(atoms, n_atoms, "_NET_WM_MOVERESIZE"))
@@ -432,9 +438,9 @@ static void gnome_wm_check_features(void)
 	if (!set_sticky_func &&
 	    find_atom(atoms, n_atoms, "_WIN_STATE"))
 		set_sticky_func = gnome_wm_set_window_sticky;
-	if (!set_skip_winlist_func &&
+	if (!set_skip_taskbar_func &&
 	    find_atom(atoms, n_atoms, "_WIN_HINTS"))
-		set_skip_winlist_func = gnome_wm_set_skip_winlist;
+		set_skip_taskbar_func = gnome_wm_set_skip_taskbar;
 
 	XFree(atoms);
 }
