@@ -1,7 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
  *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2003  Haavard Kvaalen
+ *  Copyright (C) 1999-2004  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@ GList *playlist = NULL;
 GList *shuffle_list = NULL;
 GList *queued_list = NULL;
 static gboolean playlist_get_info_scan_active = FALSE;
-static gboolean playlist_get_info_going = FALSE;
+gboolean playlist_get_info_going = FALSE;
 static pthread_t playlist_get_info_thread;
 pthread_mutex_t playlist_mutex = PTHREAD_MUTEX_INITIALIZER;
 static PlaylistEntry *playlist_position;
@@ -87,7 +87,7 @@ void playlist_clear(void)
 		node = playlist;
 		while (node)
 		{
-			entry = (PlaylistEntry *) node->data;
+			entry = node->data;
 			if (entry->filename)
 				g_free(entry->filename);
 			if (entry->title)
@@ -116,7 +116,7 @@ void playlist_delete_node(GList *node, gboolean *set_info_text, gboolean *restar
 	 */
 	if (playlist_position)
 		playing_song = g_list_find(playlist, playlist_position);
-	entry = (PlaylistEntry *) node->data;
+	entry = node->data;
 	if (playing_song == node)
 	{
 		*set_info_text = TRUE;
@@ -234,7 +234,7 @@ void playlist_delete(gboolean crop)
 
 	while (node)
 	{
-		entry = (PlaylistEntry *) node->data;
+		entry = node->data;
 		next = g_list_next(node);
 		if ((entry->selected && !crop) || (!entry->selected && crop))
 			playlist_delete_node(node, &set_info_text, &restart_playing);
@@ -288,7 +288,7 @@ static gboolean is_playlist_name(char *pathname)
 }
 
 
-void playlist_ins(gchar * filename, glong pos)
+void playlist_ins(char * filename, long pos)
 {
 	gboolean ispl = FALSE;
 
@@ -482,7 +482,7 @@ guint playlist_ins_dir(char *path, long pos, gboolean background)
 	return entries;
 }
 
-guint playlist_ins_url_string(gchar * string, glong pos)
+guint playlist_ins_url_string(char * string, long pos)
 {
 	char *temp;
 	struct stat statbuf;
@@ -597,7 +597,7 @@ void playlist_play(void)
 	playlist_check_pos_current();
 }
 
-void playlist_set_info(gchar * title, gint length, gint rate, gint freq, gint nch)
+void playlist_set_info(char *title, int length, int rate, int freq, int nch)
 {
 	PL_LOCK();
 	if (playlist_position)
@@ -608,8 +608,6 @@ void playlist_set_info(gchar * title, gint length, gint rate, gint freq, gint nc
 	}
 	PL_UNLOCK();
 	mainwin_set_song_info(rate, freq, nch);
-	mainwin_set_info_text();
-	playlistwin_update_list();
 }
 
 void playlist_check_pos_current(void)
@@ -749,49 +747,47 @@ void playlist_prev(void)
 	}
 }
 
-void playlist_queue(void)
-{
-	GList *list = playlist_get_selected();
-	GList *it = list;
- 
-	PL_LOCK();
-	while (it)
-	{
-		GList *next = it->next;
-		GList *temp;
-			
-		it->data = g_list_nth_data(playlist, GPOINTER_TO_INT(it->data));
-		temp = g_list_find(queued_list, it->data);
-		if (temp)
-		{
-			queued_list = g_list_remove_link(queued_list, temp);
-			g_list_free_1(temp);
-			list = g_list_remove_link(list, it);
-			g_list_free_1(it);
-		}
-		it = next;
-	}
-	queued_list = g_list_concat(queued_list, list);
-	PL_UNLOCK();
- 
-	playlistwin_update_list();
-}
- 
-void playlist_queue_position(gint pos)
+static void playlist_queue_toggle(void *data)
 {
 	GList *temp;
-	PlaylistEntry *entry;
- 
-	PL_LOCK();
-	entry = g_list_nth_data(playlist, pos);
-	temp = g_list_find(queued_list, entry);
+
+	temp = g_list_find(queued_list, data);
 	if (temp)
 	{
 		queued_list = g_list_remove_link(queued_list, temp);
 		g_list_free_1(temp);
 	}
 	else
-		queued_list = g_list_append(queued_list, entry);
+		queued_list = g_list_append(queued_list, data);
+}
+
+
+void playlist_queue_selected(void)
+{
+	GList *node;
+ 
+	PL_LOCK();
+	for (node = get_playlist(); node; node = node->next)
+	{
+		PlaylistEntry *entry = node->data;
+
+		if (!entry->selected)
+			continue;
+
+		playlist_queue_toggle(entry);
+	}
+	PL_UNLOCK();
+ 
+	playlistwin_update_list();
+}
+ 
+void playlist_queue_position(int pos)
+{
+	void *entry;
+ 
+	PL_LOCK();
+	entry = g_list_nth_data(playlist, pos);
+	playlist_queue_toggle(entry);
 	PL_UNLOCK();
  
 	playlistwin_update_list();
@@ -917,7 +913,7 @@ void playlist_eof_reached(void)
 	playlistwin_update_list();
 }
 
-gint get_playlist_length(void)
+int get_playlist_length(void)
 {
 	int retval;
 
@@ -1051,7 +1047,7 @@ gboolean playlist_save(char *filename, gboolean is_pls)
 	return (fclose(file) == 0);
 }
 
-gboolean playlist_load(gchar * filename)
+gboolean playlist_load(char * filename)
 {
 	return playlist_load_ins(filename, -1);
 }
@@ -1239,7 +1235,7 @@ GList *get_playlist(void)
 	return playlist;
 }
 
-gint __get_playlist_position(void)
+int __get_playlist_position(void)
 {
 	/* Caller should hold playlist_mutex */
 	if (playlist && playlist_position)
@@ -1247,7 +1243,7 @@ gint __get_playlist_position(void)
 	return 0;
 }
 
-gint get_playlist_position(void)
+int get_playlist_position(void)
 {
 	int retval;
 
@@ -1258,7 +1254,7 @@ gint get_playlist_position(void)
 	return retval;
 }
 
-gchar * playlist_get_filename(gint pos)
+char * playlist_get_filename(int pos)
 {
 	char *ret;
 	PlaylistEntry *entry;
@@ -1284,7 +1280,7 @@ gchar * playlist_get_filename(gint pos)
 	return ret;
 }
 
-gchar * playlist_get_songtitle(gint pos)
+char * playlist_get_songtitle(int pos)
 {
 	char *title = NULL, *filename;
 	PlaylistEntry *entry;
@@ -1327,7 +1323,7 @@ gchar * playlist_get_songtitle(gint pos)
 	return title;
 }
 
-gint playlist_get_songtime(gint pos)
+int playlist_get_songtime(int pos)
 {
 	int retval = -1;
 	PlaylistEntry *entry;
@@ -1894,7 +1890,7 @@ void playlist_remove_dead_files(void)
 	{
 		/* A dead file is a file that is not readable. */
 		next_node = g_list_next(node);
-		entry = (PlaylistEntry *) node->data;
+		entry = node->data;
 
 		if (entry && entry->filename &&
 		    !strstr(entry->filename, "://") && /* Don't kill URL's */
