@@ -1,7 +1,7 @@
 /*  XMMS - Cross-platform multimedia player
  *  Copyright (C) 1998-2002  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2002  Haavard Kvaalen
+ *  Copyright (C) 1999-2004  Haavard Kvaalen
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@ GtkWidget *playlistwin;
 static GtkWidget *playlistwin_url_window = NULL;
 static GtkItemFactory *playlistwin_sort_menu, *playlistwin_sub_menu;
 static GtkItemFactory *playlistwin_popup_menu, *playlistwin_save_menu;
+static GtkAccelGroup *playlistwin_accel;
 
 static GdkPixmap *playlistwin_bg;
 static GdkBitmap *playlistwin_mask;
@@ -65,15 +66,19 @@ enum
 	ADD_URL, ADD_DIR, ADD_FILE,
 	SUB_MISC, SUB_ALL, SUB_CROP, SUB_SELECTED,
 	SEL_INV, SEL_ZERO, SEL_ALL,
-	MISC_SORT, MISC_FILEINFO, MISC_MISCOPTS,
+	MISC_SORT, MISC_FILEINFO, MISC_MISCOPTS, MISC_QUEUE,
 	PLIST_NEW, PLIST_SAVE, PLIST_LOAD,
 	SEL_LOOKUP,
 };
 
-void playlistwin_sort_menu_callback(gpointer cb_data, guint action, GtkWidget * w);
-void playlistwin_sub_menu_callback(gpointer cb_data, guint action, GtkWidget * w);
-void playlistwin_save_type_cb(gpointer cb_data, guint action, GtkWidget * w);
-void playlistwin_set_hints(void);
+static void playlistwin_sort_menu_callback(gpointer cb_data, guint action, GtkWidget * w);
+static void playlistwin_sub_menu_callback(gpointer cb_data, guint action, GtkWidget * w);
+static void playlistwin_save_type_cb(gpointer cb_data, guint action, GtkWidget * w);
+static void playlistwin_set_hints(void);
+static void playlistwin_set_shade(gboolean shaded);
+static void playlistwin_popup_menu_callback(gpointer cb_data, guint action, GtkWidget * w);
+static void playlistwin_draw_frame(void);
+
 
 enum
 {
@@ -145,11 +150,10 @@ static const int playlistwin_sub_menu_entries_num =
 	sizeof(playlistwin_sub_menu_entries) / 
 	sizeof(playlistwin_sub_menu_entries[0]);
 
-void playlistwin_popup_menu_callback(gpointer cb_data, guint action, GtkWidget * w);
-
 GtkItemFactoryEntry playlistwin_popup_menu_entries[] =
 {
 	{N_("/View File Info"), NULL, playlistwin_popup_menu_callback, MISC_FILEINFO, "<Item>"},
+	{N_("/Queue - Unqueue"), "Q", playlistwin_popup_menu_callback, MISC_QUEUE, "<Item>"},
 	{N_("/-"), NULL, NULL, 0, "<Separator>"},
 	{N_("/Add"), NULL, NULL, 0, "<Branch>"},
 	{N_("/Add/File"), NULL, playlistwin_popup_menu_callback, ADD_FILE, "<Item>"},
@@ -181,8 +185,6 @@ static const int playlistwin_popup_menu_entries_num =
 	sizeof(playlistwin_popup_menu_entries) / 
 	sizeof(playlistwin_popup_menu_entries[0]);
 
-void playlistwin_draw_frame(void);
-
 static void playlistwin_update_info(void)
 {
 	char *text, *sel_text, *tot_text;
@@ -216,7 +218,7 @@ static void playlistwin_update_info(void)
 	g_free(sel_text);
 }
 
-void playlistwin_update_sinfo(void)
+static void playlistwin_update_sinfo(void)
 {
 	char *posstr, *timestr, *title, *info, *dots;
 	int pos, time, max_len;
@@ -284,14 +286,15 @@ void playlistwin_update_sinfo(void)
 	g_free(info);
 }
 
-gboolean playlistwin_item_visible(gint index)
+gboolean playlistwin_item_visible(int index)
 {
-	if (index >= playlistwin_list->pl_first && index < (playlistwin_list->pl_first + playlistwin_list->pl_num_visible))
+	if (index >= playlistwin_list->pl_first &&
+	    index < (playlistwin_list->pl_first + playlistwin_list->pl_num_visible))
 		return TRUE;
 	return FALSE;
 }
 
-gint playlistwin_get_toprow(void)
+int playlistwin_get_toprow(void)
 {
 	if (playlistwin_list)
 		return (playlistwin_list->pl_first);
@@ -315,7 +318,7 @@ void playlistwin_update_list(void)
 	playlistwin_update_sinfo();
 }
 
-void playlistwin_create_mask(void)
+static void playlistwin_create_mask(void)
 {
 	GdkBitmap *tmp;
 	GdkGC *gc;
@@ -342,8 +345,9 @@ void playlistwin_set_shade_menu_cb(gboolean shaded)
 	cfg.playlist_shaded = shaded;
 	if (!cfg.show_wm_decorations)
 		dock_shade(dock_window_list, playlistwin, PLAYLIST_HEIGHT);
-	else 
-		gtk_window_set_default_size(GTK_WINDOW(playlistwin), cfg.playlist_width, PLAYLIST_HEIGHT);
+	else
+		gtk_window_set_default_size(GTK_WINDOW(playlistwin),
+					    cfg.playlist_width, PLAYLIST_HEIGHT);
 
 	if (shaded)
 	{
@@ -371,12 +375,12 @@ void playlistwin_set_shade_menu_cb(gboolean shaded)
 	draw_playlist_window(TRUE);
 }
 
-void playlistwin_shade_toggle(void)
+static void playlistwin_shade_toggle(void)
 {
 	playlistwin_set_shade(!cfg.playlist_shaded);
 }
 
-void playlistwin_set_shade(gboolean shaded)
+static void playlistwin_set_shade(gboolean shaded)
 {
 	GtkWidget *widget;
 	widget = gtk_item_factory_get_widget(mainwin_options_menu,
@@ -390,7 +394,7 @@ void playlistwin_raise(void)
 		gdk_window_raise(playlistwin->window);
 }
 
-void playlistwin_release(GtkWidget * widget, GdkEventButton * event, gpointer callback_data)
+static void playlistwin_release(GtkWidget * widget, GdkEventButton * event, gpointer callback_data)
 {
 	if (event->button == 3)
 		return;
@@ -398,17 +402,9 @@ void playlistwin_release(GtkWidget * widget, GdkEventButton * event, gpointer ca
 	gdk_pointer_ungrab(GDK_CURRENT_TIME);
 	gdk_flush();
 	if (playlistwin_resizing)
-	{
 		playlistwin_resizing = FALSE;
-	}
 	else if (dock_is_moving(playlistwin))
-	{
 		dock_move_release(playlistwin);
-#if 0
-		if(cfg.playlist_transparent)
-			playlistwin_update_list();
-#endif
-	}
 	else
 	{
 		handle_release_cb(playlistwin_wlist, widget, event);
@@ -423,36 +419,36 @@ void playlistwin_scroll(int num)
 	playlistwin_update_list();
 }
 
-void playlistwin_scroll_up_pushed(void)
+static void playlistwin_scroll_up_pushed(void)
 {
 	playlistwin_list->pl_first -= 3;
 	playlistwin_update_list();
 }
 
-void playlistwin_scroll_down_pushed(void)
+static void playlistwin_scroll_down_pushed(void)
 {
 	playlistwin_list->pl_first += 3;
 	playlistwin_update_list();
 }
 
-void playlistwin_select_all(void)
+static void playlistwin_select_all(void)
 {
 	playlist_select_all(TRUE);
-	playlistwin_list->pl_prev_selected=0;
-	playlistwin_list->pl_prev_min=0;
-	playlistwin_list->pl_prev_max=get_playlist_length()-1;
+	playlistwin_list->pl_prev_selected = 0;
+	playlistwin_list->pl_prev_min = 0;
+	playlistwin_list->pl_prev_max = get_playlist_length()-1;
 	playlistwin_update_list();
 }
 
-void playlistwin_select_none(void)
+static void playlistwin_select_none(void)
 {
 	playlist_select_all(FALSE);
-	playlistwin_list->pl_prev_selected=-1;
-	playlistwin_list->pl_prev_min=-1;
+	playlistwin_list->pl_prev_selected = -1;
+	playlistwin_list->pl_prev_min = -1;
 	playlistwin_update_list();
 }
 
-void playlistwin_inverse_selection(void)
+static void playlistwin_inverse_selection(void)
 {
 	playlist_select_invert_all();
 	playlistwin_list->pl_prev_selected=-1;
@@ -462,7 +458,7 @@ void playlistwin_inverse_selection(void)
 
 static void playlistwin_resize(int width, int height)
 {
-	gint bx, by, nw, nh;
+	int bx, by, nw, nh;
 	GdkPixmap *oldbg;
 	gboolean dummy;
 
@@ -470,7 +466,7 @@ static void playlistwin_resize(int width, int height)
 	nw = (bx * 25) + 275;
 	if (nw < 275)
 		nw = 275;
-	
+
 	if (!cfg.playlist_shaded)
 	{
 		by = (height - 58) / 29;
@@ -480,12 +476,13 @@ static void playlistwin_resize(int width, int height)
 	}
 	else
 		nh = cfg.playlist_height;
-	
+
 	if (nw == cfg.playlist_width && nh == cfg.playlist_height)
 		return;
 
 	cfg.playlist_width = nw;
 	cfg.playlist_height = nh;
+
 	resize_widget(playlistwin_list, cfg.playlist_width - 31, cfg.playlist_height - 58);
 	move_widget(playlistwin_slider, cfg.playlist_width - 15, 20);
 	resize_widget(playlistwin_sinfo, cfg.playlist_width - 35, 14);
@@ -549,7 +546,7 @@ static void playlistwin_queue_resize(int width, int height)
 	playlistwin_resizeq.h = height;
 }
 
-void playlistwin_motion(GtkWidget * widget, GdkEventMotion * event, gpointer callback_data)
+static void playlistwin_motion(GtkWidget * widget, GdkEventMotion * event, gpointer callback_data)
 {
 	XEvent ev;
 
@@ -579,7 +576,7 @@ void playlistwin_motion(GtkWidget * widget, GdkEventMotion * event, gpointer cal
 	while (XCheckMaskEvent(GDK_DISPLAY(), ButtonMotionMask, &ev)) ;
 }
 
-void playlistwin_show_filebrowser(void)
+static void playlistwin_show_filebrowser(void)
 {
 	static GtkWidget *filebrowser;
 	if (filebrowser != NULL)
@@ -592,9 +589,9 @@ void playlistwin_show_filebrowser(void)
 
 
 
-void playlistwin_url_ok_clicked(GtkWidget * w, GtkWidget * entry)
+static void playlistwin_url_ok_clicked(GtkWidget * w, GtkWidget * entry)
 {
-	gchar *text, *temp;
+	char *text, *temp;
 
 	text = gtk_entry_get_text(GTK_ENTRY(entry));
 	if (text && *text)
@@ -622,14 +619,14 @@ void playlistwin_show_add_url_window(void)
 
 }
 
-void playlistwin_add_dir_handler(gchar * dir)
+static void playlistwin_add_dir_handler(char * dir)
 {
 	g_free(cfg.filesel_path);
 	cfg.filesel_path = g_strdup(dir);
 	playlist_add_dir(dir);
 }
 
-void playlistwin_show_dirbrowser(void)
+static void playlistwin_show_dirbrowser(void)
 {
 	static GtkWidget *dir_browser;
 
@@ -664,6 +661,26 @@ static void playlistwin_fileinfo(void)
 		playlist_fileinfo_current();
 }
 
+static void playlistwin_set_sensitive_popupmenu(void)
+{
+	GtkWidget *w;
+	gboolean set;
+
+	set = playlist_get_num_selected() > 0;
+
+	w = gtk_item_factory_get_widget(playlistwin_popup_menu,
+					"/Remove/Selected");
+	gtk_widget_set_sensitive(w, set);
+
+	w = gtk_item_factory_get_widget(playlistwin_popup_menu,
+					"/Selection/Read Extended Info");
+	gtk_widget_set_sensitive(w, set);
+
+	w = gtk_item_factory_get_widget(playlistwin_sub_menu,
+					"/Physically Delete Files");
+	gtk_widget_set_sensitive(w, set);
+}
+
 static void playlistwin_set_sensitive_sortmenu(void)
 {
 	GtkWidget *w;
@@ -685,7 +702,7 @@ static void playlistwin_set_sensitive_sortmenu(void)
 	gtk_widget_set_sensitive(w, set);
 }
 
-void playlistwin_save_playlist_error(char* path, GtkWidget *filesel)
+static void playlistwin_save_playlist_error(char* path, GtkWidget *filesel)
 {
 	GtkWidget *dialog, *label, *bbox, *close;
 	char *text;
@@ -744,7 +761,7 @@ static void playlistwin_check_overwrite(GtkWidget *filesel, char *filename, int
 	gtk_window_set_title(GTK_WINDOW(dialog), _("File exists!"));
 	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(filesel));
 
-	text = g_strdup_printf(_("%s already exist."), filename);
+	text = g_strdup_printf(_("%s already exists."), filename);
 	label = gtk_label_new(text);
 	gtk_misc_set_padding(GTK_MISC(label), 10, 10);
 	g_free(text);
@@ -778,7 +795,7 @@ static void playlistwin_check_overwrite(GtkWidget *filesel, char *filename, int
 	gtk_widget_show_all(dialog);
 }
 
-void playlistwin_save_filesel_ok(GtkWidget * w, GtkFileSelection * filesel)
+static void playlistwin_save_filesel_ok(GtkWidget * w, GtkFileSelection * filesel)
 {
 	char *filename, *slash;
 	struct stat statd;
@@ -812,12 +829,12 @@ void playlistwin_save_filesel_ok(GtkWidget * w, GtkFileSelection * filesel)
 		{
 			char *m;
 			GtkWidget *w;
-			m = g_strdup_printf(_("Unknown filetype for %s"),
+			m = g_strdup_printf(_("Unknown file type for %s"),
 					    filename);
 			w = xmms_show_message(_("Unable to save playlist"), m,
 					      _("Ok"), TRUE, NULL, NULL);
-			gtk_window_set_transient_for(GTK_WINDOW(filesel),
-						     GTK_WINDOW(w));
+			gtk_window_set_transient_for(GTK_WINDOW(w),
+						     GTK_WINDOW(filesel));
 
 			g_free(m);
 			return;
@@ -840,9 +857,9 @@ void playlistwin_save_filesel_ok(GtkWidget * w, GtkFileSelection * filesel)
 	}
 }
 
-void playlistwin_load_filesel_ok(GtkWidget * w, GtkWidget * filesel)
+static void playlistwin_load_filesel_ok(GtkWidget * w, GtkWidget * filesel)
 {
-	gchar *filename, *text, *tmp;
+	char *filename, *text, *tmp;
 
 	if (util_filebrowser_is_dir(GTK_FILE_SELECTION(filesel)))
 		return;
@@ -869,7 +886,7 @@ void playlistwin_load_filesel_ok(GtkWidget * w, GtkWidget * filesel)
 
 static void playlistwin_show_sub_misc_menu(void)
 {
-	gint x, y;
+	int x, y;
 	GtkWidget *widget;
 
 	widget = gtk_item_factory_get_widget(playlistwin_sub_menu,
@@ -938,6 +955,10 @@ static void playlistwin_show_save_filesel(void)
 				  GTK_OBJECT(filesel));
 	gtk_signal_connect(GTK_OBJECT(filesel), "destroy",
 			   GTK_SIGNAL_FUNC(gtk_widget_destroyed), &filesel);
+	/*
+	 * I18N: "Save options" here is "options for saving, not "save
+	 * the options"
+	 */
 	frame = gtk_frame_new(_("Save options"));
 	hbox = gtk_hbox_new(FALSE, 5);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
@@ -954,12 +975,12 @@ static void playlistwin_show_save_filesel(void)
 	gtk_widget_show_all(filesel);
 }
 
-void playlistwin_save_type_cb(gpointer cb_data, guint action, GtkWidget * w)
+static void playlistwin_save_type_cb(gpointer cb_data, guint action, GtkWidget * w)
 {
 	playlistwin_save_type = action;
 }
 
-void playlistwin_popup_handler(gint item)
+static void playlistwin_popup_handler(int item)
 {
 	switch (item)
 	{
@@ -1003,7 +1024,7 @@ void playlistwin_popup_handler(gint item)
 
 		/* Misc button */
 		case MISC_SORT: {
-			gint x, y;
+			int x, y;
 			GtkItemFactory *f;
 			playlistwin_set_sensitive_sortmenu();
 			gdk_window_get_pointer(NULL, &x, &y, NULL);
@@ -1032,7 +1053,7 @@ void playlistwin_popup_handler(gint item)
 	}
 }
 
-static gboolean inside_sensitive_widgets(gint x, gint y)
+static gboolean inside_sensitive_widgets(int x, int y)
 {
 	return (inside_widget(x, y, playlistwin_list) ||
 		inside_widget(x, y, playlistwin_slider) ||
@@ -1062,7 +1083,7 @@ static gboolean inside_sensitive_widgets(gint x, gint y)
  event->y >= cfg.playlist_height - (y1) &&	\
  event->y < cfg.playlist_height - (y2))
 
-void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer callback_data)
+static void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer callback_data)
 {
 	gboolean grab = TRUE;
 	int xpos, ypos;
@@ -1173,7 +1194,7 @@ void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer call
 		}
 		else if (event->button == 3)
 		{
-			gint mx, my;
+			int mx, my;
 			GdkModifierType modmask;
 
 			gdk_window_get_pointer(NULL, &mx, &my, &modmask);
@@ -1185,7 +1206,7 @@ void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer call
 		 !inside_sensitive_widgets(event->x, event->y) &&
 		 (cfg.easy_move || event->y < 14))
 	{
-		if (0 && hint_move_resize_available())
+		if (hint_move_resize_available())
 		{
 			hint_move_resize(playlistwin, event->x_root,
 					 event->y_root, TRUE);
@@ -1220,7 +1241,8 @@ void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer call
 		 * Pop up the main menu a few pixels down to avoid
 		 * anything to be selected initially.
 		 */
-		util_item_factory_popup(mainwin_general_menu, event->x_root, event->y_root + 2, 3, event->time);
+		util_item_factory_popup(mainwin_general_menu, event->x_root,
+					event->y_root + 2, 3, event->time);
 		grab = FALSE;
 	}
 	else if (event->button == 3 &&
@@ -1230,12 +1252,20 @@ void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer call
 		GtkWidget *w;
 		pos = playlist_list_get_playlist_position(playlistwin_list,
 							  event->x, event->y);
+		if (pos != -1)
+			pos++;
 		sensitive = pos != -1;
 		w = gtk_item_factory_get_widget(playlistwin_popup_menu,
 						"/View File Info");
 		gtk_widget_set_sensitive(w, sensitive);
 
+		w = gtk_item_factory_get_widget(playlistwin_popup_menu,
+						"/Queue - Unqueue");
+		gtk_widget_set_sensitive(w, sensitive);
+
 		playlistwin_set_sensitive_sortmenu();
+		playlistwin_set_sensitive_popupmenu();
+	
 		util_item_factory_popup_with_data(playlistwin_popup_menu,
 						  GINT_TO_POINTER(pos), NULL,
 						  event->x_root,
@@ -1259,7 +1289,7 @@ void playlistwin_press(GtkWidget * widget, GdkEventButton * event, gpointer call
 
 }
 
-void playlistwin_focus_in(GtkWidget * widget, GdkEvent * event, gpointer callback_data)
+static void playlistwin_focus_in(GtkWidget * widget, GdkEvent * event, gpointer callback_data)
 {
 	playlistwin_close->pb_allow_draw = TRUE;
 	playlistwin_shade->pb_allow_draw = TRUE;
@@ -1267,7 +1297,7 @@ void playlistwin_focus_in(GtkWidget * widget, GdkEvent * event, gpointer callbac
 	draw_playlist_window(TRUE);
 }
 
-void playlistwin_focus_out(GtkWidget * widget, GdkEventButton * event, gpointer callback_data)
+static void playlistwin_focus_out(GtkWidget * widget, GdkEventButton * event, gpointer callback_data)
 {
 	playlistwin_close->pb_allow_draw = FALSE;
 	playlistwin_shade->pb_allow_draw = FALSE;
@@ -1283,7 +1313,7 @@ static gboolean playlistwin_configure(GtkWidget * window, GdkEventConfigure *eve
 	if (cfg.show_wm_decorations || hint_move_resize_available())
 	{
 		if (event->width != cfg.playlist_width ||
-		    event->height != cfg.playlist_height)
+		    (event->height != cfg.playlist_height && event->height != 14))
 			playlistwin_queue_resize(event->width, event->height);
 	}
 	
@@ -1304,7 +1334,7 @@ void playlistwin_set_back_pixmap()
 	gdk_window_clear(playlistwin->window);
 }
 
-gint playlistwin_client_event(GtkWidget *w,GdkEventClient *event, gpointer data)
+static int playlistwin_client_event(GtkWidget *w, GdkEventClient *event, gpointer data)
 {
 	static GdkAtom atom_rcfiles = GDK_NONE;
 
@@ -1321,13 +1351,13 @@ gint playlistwin_client_event(GtkWidget *w,GdkEventClient *event, gpointer data)
 	return FALSE;
 }
 
-gint playlistwin_delete(GtkWidget * w, gpointer data)
+static int playlistwin_delete(GtkWidget * w, gpointer data)
 {
 	playlistwin_show(FALSE);
 	return TRUE;
 }
 
-void playlistwin_set_hints(void)
+static void playlistwin_set_hints(void)
 {
 	GdkGeometry geometry;
 	GdkWindowHints mask;
@@ -1359,10 +1389,10 @@ void playlistwin_set_hints(void)
 				      playlistwin, &geometry, mask);
 }
 
-void playlistwin_physically_delete_cb(GtkWidget *widget, gpointer data)
+static void playlistwin_physically_delete_cb(GtkWidget *widget, gpointer data)
 {
 	GList *node, *selected_list = data;
-	gint deleted = 0, length;
+	int deleted = 0, length;
 	char *message = NULL;
 
 	length = g_list_length(selected_list);
@@ -1400,12 +1430,12 @@ void playlistwin_physically_delete_cb(GtkWidget *widget, gpointer data)
 	}
 }
 	
-void playlistwin_physically_delete(void)
+static void playlistwin_physically_delete(void)
 {
 	GtkWidget *dialog, *vbox, *label, *bbox, *ok, *cancel;
 	GList *node, *selected_list = NULL;
 	char *text;
-	gint length;
+	int length;
 
 	PL_LOCK();
 	for (node = get_playlist(); node != NULL; node = g_list_next(node))
@@ -1498,10 +1528,14 @@ static void playlistwin_keypress_up_down_handler(PlayList_List *pl, gboolean up,
 			playlist_list_move_up(pl);
 		else
 			playlist_list_move_down(pl);
-		if (pl->pl_prev_min < pl->pl_first)
-			pl->pl_first = pl->pl_prev_min;
-		else if (pl->pl_prev_max >= (pl->pl_first + pl->pl_num_visible))
-			pl->pl_first = pl->pl_prev_max - pl->pl_num_visible + 1;
+		
+		if (pl->pl_prev_min != -1)
+		{
+			if (pl->pl_prev_min < pl->pl_first)
+				pl->pl_first = pl->pl_prev_min;
+			else if (pl->pl_prev_max >= (pl->pl_first + pl->pl_num_visible))
+				pl->pl_first = pl->pl_prev_max - pl->pl_num_visible + 1;
+		}
 		return;
 	}
 	else if (up)
@@ -1522,7 +1556,7 @@ static void playlistwin_keypress_up_down_handler(PlayList_List *pl, gboolean up,
 
 }
 
-gboolean playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
+static gboolean playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
 {
 	guint keyval;
 	gboolean refresh = TRUE;
@@ -1565,11 +1599,6 @@ gboolean playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
 			}
 			refresh = FALSE;
 			break;
-		case GDK_3:
-			if (event->state & GDK_CONTROL_MASK)
-				playlistwin_fileinfo();
-			refresh = FALSE;
-			break;
 		case GDK_Delete:
 			if (event->state & GDK_CONTROL_MASK)
 				playlist_delete(TRUE);
@@ -1587,7 +1616,8 @@ gboolean playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
 			refresh=FALSE;
 			break;
 		default:
-			gtk_widget_event(mainwin, (GdkEvent *) event);
+			if (!gtk_accel_group_activate(playlistwin_accel, event->keyval, event->state))
+				gtk_widget_event(mainwin, (GdkEvent *) event);
 			refresh = FALSE;
 			break;
 	}
@@ -1597,9 +1627,9 @@ gboolean playlistwin_keypress(GtkWidget * w, GdkEventKey * event, gpointer data)
 	return TRUE;
 }
 
-void playlistwin_draw_frame(void)
+static void playlistwin_draw_frame(void)
 {
-	gint w, h, y, i, c;
+	int w, h, y, i, c;
 	SkinIndex src;
 
 	w = cfg.playlist_width;
@@ -1704,7 +1734,7 @@ void draw_playlist_window(gboolean force)
 	unlock_widget_list(playlistwin_wlist);
 }
 
-void playlistwin_sort_menu_callback(gpointer cb_data, guint action, GtkWidget * w)
+static void playlistwin_sort_menu_callback(gpointer cb_data, guint action, GtkWidget * w)
 {
 	switch (action)
 	{
@@ -1751,7 +1781,7 @@ void playlistwin_sort_menu_callback(gpointer cb_data, guint action, GtkWidget *
 	}
 }
 
-void playlistwin_sub_menu_callback(gpointer cb_data, guint action, GtkWidget * w)
+static void playlistwin_sub_menu_callback(gpointer cb_data, guint action, GtkWidget * w)
 {
 	switch (action)
 	{
@@ -1785,9 +1815,9 @@ void playlistwin_vis_disable(void)
 	draw_playlist_window(TRUE);
 }
 
-void playlistwin_set_time(gint time, gint length, TimerMode mode)
+void playlistwin_set_time(int time, int length, TimerMode mode)
 {
-	gchar *text, sign;
+	char *text, sign;
 
 	if (mode == TIMER_REMAINING && length != -1)
 	{
@@ -1831,14 +1861,14 @@ static void playlistwin_drag_data_received(GtkWidget * widget,
 			pos = ((y - ((Widget *) playlistwin_list)->y) / playlistwin_list->pl_fheight) + playlistwin_list->pl_first;
 			if (pos > get_playlist_length())
 				pos = get_playlist_length();
-			playlist_ins_url_string((gchar *) selection_data->data, pos);
+			playlist_ins_url_string(selection_data->data, pos);
 		}
 		else
-			playlist_add_url_string((gchar *) selection_data->data);
+			playlist_add_url_string(selection_data->data);
 	}
 }
 
-void playlistwin_close_cb(void)
+static void playlistwin_close_cb(void)
 {
 	playlistwin_show(FALSE);
 }
@@ -1927,6 +1957,8 @@ void playlistwin_create(void)
 {
 	GtkWidget *item, *menu;
 
+	playlistwin_accel = gtk_accel_group_new();	
+
 	playlistwin_sort_menu = gtk_item_factory_new(GTK_TYPE_MENU,
 						     "<Main>", NULL);
 	gtk_item_factory_set_translate_func(playlistwin_sort_menu,
@@ -1945,7 +1977,7 @@ void playlistwin_create(void)
 					gdk_rgb_get_visual()->depth);
 
 	playlistwin_popup_menu =
-		gtk_item_factory_new(GTK_TYPE_MENU, "<Main>", NULL);
+		gtk_item_factory_new(GTK_TYPE_MENU, "<Main>", playlistwin_accel);
 	gtk_item_factory_set_translate_func(playlistwin_popup_menu,
 					    util_menu_translate, NULL, NULL);
 	gtk_item_factory_create_items(GTK_ITEM_FACTORY(playlistwin_popup_menu),
@@ -2020,14 +2052,24 @@ void playlistwin_real_hide(void)
 	tbutton_set_toggled(mainwin_pl, FALSE);
 }
 
-void playlistwin_popup_menu_callback(gpointer cb_data, guint action, GtkWidget * w)
+static void playlistwin_popup_menu_callback(gpointer cb_data, guint action, GtkWidget * w)
 {
 	int pos = GPOINTER_TO_INT(gtk_item_factory_popup_data_from_widget(w));
 	switch (action)
 	{
 		case MISC_FILEINFO:
-			if (pos != -1)
-				playlist_fileinfo(pos);
+			if (pos == 0)
+				/* We got here via an accelerator */
+				playlistwin_fileinfo();
+			else if (pos != -1)
+				playlist_fileinfo(pos - 1);
+			break;
+		case MISC_QUEUE:
+			if (pos == 0)
+				/* We got here via an accelerator */
+				playlist_queue_selected();
+			else if (pos != -1)
+				playlist_queue_position(pos - 1);
 			break;
 		case SEL_LOOKUP:
 			playlist_read_info_selection();
